==> v0.1.0.tar.gz: quartet-rnaseq-report/src/tservice/plugins/quartet_rnaseq_report.clj <==
(ns tservice.plugins.quartet-rnaseq-report
  (:require [clojure.data.json :as json]
            [clojure.spec.alpha :as s]
            [clojure.tools.logging :as log]
            [spec-tools.core :as st]
            [clojure.java.shell :as shell :refer [sh]]
            [tservice.lib.files :as ff]
            [tservice.lib.fs :as fs-lib]
            [tservice.vendor.multiqc :as mq]
            [tservice.api.config :refer [add-env-to-path]]
            [tservice.api.storage.fs :as fs-api]
            [tservice.api.data.csv :refer [write-csv!]]
            [tservice.api.task :refer [publish-event! make-plugin-metadata make-events-init create-task! update-process!]]
            [tservice.plugins.quartet-rnaseq-report.exp2qcdt :as exp2qcdt]
            [tservice.plugins.quartet-rnaseq-report.merge-exp :as me]
            [tservice.plugins.quartet-rnaseq-report.util :refer [gen-multiqc-config]]))

;;; ------------------------------------------------ Event Specs ------------------------------------------------
(s/def ::name
  (st/spec
   {:spec                string?
    :type                :string
    :description         "The name of the report"
    :swagger/default     ""
    :reason              "Not a valid report name"}))

(s/def ::description
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Description of the report"
    :swagger/default     ""
    :reason              "Not a valid description."}))

(s/def ::filepath
  (st/spec
   {:spec                (s/and string? #(re-matches #"^oss:\/\/[a-zA-Z0-9_\/]+.*" %))
    :type                :string
    :description         "File path for covertor."
    :swagger/default     nil
    :reason              "The filepath must be string."}))

(s/def ::group
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A group name which is matched with library."
    :swagger/default     []
    :reason              "The group must a string."}))

(s/def ::library
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A library name."
    :swagger/default     []
    :reason              "The library must a string."}))

(s/def ::sample
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A sample name."
    :swagger/default     []
    :reason              "The sample name must a string."}))

(s/def ::metadat-item
  (s/keys :req-un [::library
                   ::group
                   ::sample]))

(s/def ::metadata
  (s/coll-of ::metadat-item))

(s/def ::lab
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Lab name."
    :swagger/default     []
    :reason              "The lab_name must be string."}))

(s/def ::sequencing_platform
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Sequencing Platform."
    :swagger/default     []
    :reason              "The sequencing_platform must be string."}))

(s/def ::sequencing_method
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Sequencing Method"
    :swagger/default     []
    :reason              "The sequencing_method must be string."}))

(s/def ::library_protocol
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Library protocol."
    :swagger/default     []
    :reason              "The library_protocol must be string."}))

(s/def ::library_kit
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Library kit."
    :swagger/default     []
    :reason              "The library_kit must be string."}))

(s/def ::read_length
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Read length"
    :swagger/default     []
    :reason              "The read_length must be string."}))

(s/def ::date
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Date"
    :swagger/default     []
    :reason              "The date must be string."}))

(s/def ::parameters
  (s/keys :req-un [::lab
                   ::sequencing_platform
                   ::sequencing_method
                   ::library_protocol
                   ::library_kit
                   ::read_length
                   ::date]))

(def quartet-rna-report-params-body
  "A spec for the body parameters."
  (s/keys :req-un [::name ::description ::filepath ::metadata ::parameters]
          :opt-un [::description]))

;;; ------------------------------------------------ Event Metadata ------------------------------------------------
(def metadata
  (make-plugin-metadata
   {:name "quartet-rnaseq-report"
    :summary "Generate the QC Report for Quartet RNA-Seq data."
    :params-schema quartet-rna-report-params-body
    :handler (fn [{:keys [name description filepath metadata parameters] :as payload}]
               (let [workdir (ff/get-workdir)
                     report-file (fs-lib/join-paths workdir "multireport.html")
                     log-path (fs-lib/join-paths workdir "log")
                     response {:report report-file
                               :log log-path
                               :response-type :data2report}
                     task-id (create-task! {:name name
                                            :description (or description "Generate the QC Report for Quartet RNA-Seq data.")
                                            :payload payload
                                            :plugin-name "quartet-rnaseq-report"
                                            :plugin-type "ReportPlugin"
                                            :plugin-version "v0.1.0"
                                            :response response})]
                 (fs-lib/create-directories! workdir)
                 (spit log-path (json/write-str {:status "Running" :msg ""}))
                 (update-process! task-id 0)
                 (publish-event! "quartet_rnaseq_report"
                                 {:data-dir filepath
                                  :parameters parameters
                                  :metadata metadata
                                  :dest-dir workdir
                                  :task-id task-id})
                 response))
    :plugin-type :ReportPlugin
    :response-type :data2report}))

;;; ------------------------------------------------ Event Processing ------------------------------------------------

(defn- decompression-tar
  [filepath]
  (shell/with-sh-env {:PATH   (add-env-to-path "quartet-rnaseq-report")
                      :LC_ALL "en_US.utf-8"
                      :LANG   "en_US.utf-8"}
    (let [command ["bash" "-c"
                   (format "tar -xvf %s -C %s" filepath (fs-lib/parent-path filepath))]
          result  (apply sh command)
          status (if (= (:exit result) 0) "Success" "Error")
          msg (str (:out result) "\n" (:err result))]
      {:status status
       :msg msg})))

(defn- filter-mkdir-copy
  [fmc-datadir fmc-patterns fmc-destdir fmc-newdir]
  (let [files-keep (fs-api/batch-filter-files fmc-datadir fmc-patterns)
        files-keep-dir (fs-lib/join-paths fmc-destdir fmc-newdir)]
    (fs-lib/create-directories! files-keep-dir)
    (fs-api/copy-files! files-keep files-keep-dir {:replace-existing true})))

(defn- quartet-rnaseq-report!
  "Chaining Pipeline: filter-files -> copy-files -> merge_exp_file -> exp2qcdt -> multiqc."
  [datadir parameters metadata dest-dir task-id]
  (log/info "Generate quartet rnaseq report: " datadir parameters metadata dest-dir)
  (let [metadata-file (fs-lib/join-paths dest-dir
                                         "results"
                                         "metadata.csv")
        parameters-file (fs-lib/join-paths dest-dir
                                           "results"
                                           "general-info.json")
        ballgown-dir (fs-lib/join-paths dest-dir "ballgown")
        count-dir (fs-lib/join-paths dest-dir "count")
        exp-fpkm-filepath (fs-lib/join-paths dest-dir "fpkm.csv")
        exp-count-filepath (fs-lib/join-paths dest-dir "count.csv")
        result-dir (fs-lib/join-paths dest-dir "results")
        log-path (fs-lib/join-paths dest-dir "log")
        config-path (fs-lib/join-paths dest-dir "results", "quartet_rnaseq_report.yaml")]
    (try
      (fs-lib/create-directories! result-dir)
      (log/info "Merge these files")
      (log/info "Merge gene experiment files from ballgown directory to a experiment table: " ballgown-dir exp-fpkm-filepath)
      (log/info "Merge these files")
      (log/info "Merge gene experiment files from count directory to a experiment table: " count-dir exp-count-filepath)
      (filter-mkdir-copy datadir [".*call-ballgown/.*.txt"] dest-dir "ballgown")
      (filter-mkdir-copy datadir [".*call-count/.*gene_count_matrix.csv"] dest-dir "count")
      (filter-mkdir-copy datadir [".*call-qualimapBAMqc/.*tar.gz"] dest-dir "results/post_alignment_qc/bam_qc")
      (filter-mkdir-copy datadir [".*call-qualimapRNAseq/.*tar.gz"] dest-dir "results/post_alignment_qc/rnaseq_qc")
      (filter-mkdir-copy datadir [".*call-fastqc/.*.zip"] dest-dir "results/rawqc/fastqc")
      (filter-mkdir-copy datadir [".*call-fastqscreen/.*.txt"] dest-dir "results/rawqc/fastq_screen")
      (update-process! task-id 10)
      (me/merge-exp-files! (fs-api/list-files ballgown-dir {:mode "file"}) exp-fpkm-filepath)
      (me/merge-exp-files! (fs-api/list-files count-dir {:mode "file"}) exp-count-filepath)
      (spit parameters-file (json/write-str parameters))
      (write-csv! metadata-file metadata)
      ;;(decompression-tar files-qualimap-bam)
      ;;(decompression-tar files-qualimap-RNA)
      (doseq [files-qualimap-bam-tar (fs-api/batch-filter-files (fs-lib/join-paths dest-dir "results/post_alignment_qc/bam_qc") [".*tar.gz"])]
        (decompression-tar files-qualimap-bam-tar))
      (doseq [files-qualimap-RNA-tar (fs-api/batch-filter-files (fs-lib/join-paths dest-dir "results/post_alignment_qc/rnaseq_qc") [".*tar.gz"])]
        (decompression-tar files-qualimap-RNA-tar))
      (update-process! task-id 50)
      (gen-multiqc-config config-path)
      (let [exp2qcdt-result (exp2qcdt/call-exp2qcdt! exp-fpkm-filepath exp-count-filepath metadata-file result-dir)
            multiqc-result (if (= (:status exp2qcdt-result) "Success")
                             (mq/multiqc result-dir dest-dir {:config config-path :template "default" :title "Quartet RNA report"})
                             ;;(mq/multiqc result-dir)
                             exp2qcdt-result)
            result {:status (:status multiqc-result)
                    :msg (:msg multiqc-result)}
            log (json/write-str result)]
        (log/info "Status: " result)
        (spit log-path log))
      (update-process! task-id 100)
      (catch Exception e
        (update-process! task-id -1)
        (let [log (json/write-str {:status "Error" :msg (.toString e)})]
          (log/info "Status: " log)
          (spit log-path log))))))

;;; --------------------------------------------------- Lifecycle ----------------------------------------------------
(def events-init
  "Automatically called during startup; start event listener for quartet_rnaseq_report events."
  (make-events-init "quartet_rnaseq_report" quartet-rnaseq-report!))

==> v0.1.4.tar.gz: quartet-rnaseq-report/src/tservice/plugins/quartet_rnaseq_report.clj <==
(ns tservice.plugins.quartet-rnaseq-report
  (:require [clojure.data.json :as json]
            [clojure.spec.alpha :as s]
            [clojure.tools.logging :as log]
            [spec-tools.core :as st]
            [clojure.java.shell :as shell :refer [sh]]
            [tservice.lib.files :as ff]
            [tservice.lib.fs :as fs-lib]
            [tservice.vendor.multiqc :as mq]
            [tservice.api.config :refer [add-env-to-path]]
            [tservice.api.storage.fs :as fs-api]
            [tservice.api.data.csv :refer [write-csv!]]
            [tservice.api.task :refer [publish-event! make-plugin-metadata make-events-init create-task! update-process!]]
            [tservice.plugins.quartet-rnaseq-report.exp2qcdt :as exp2qcdt]
            [tservice.plugins.quartet-rnaseq-report.merge-exp :as me]
            [tservice.plugins.quartet-rnaseq-report.util :refer [gen-multiqc-config]]))

;;; ------------------------------------------------ Event Specs ------------------------------------------------
(s/def ::name
  (st/spec
   {:spec                string?
    :type                :string
    :description         "The name of the report"
    :swagger/default     ""
    :reason              "Not a valid report name"}))

(s/def ::description
  (st/spec
   {:spec                string?
    :type                :string
    :description         "Description of the report"
    :swagger/default     ""
    :reason              "Not a valid description."}))

(s/def ::filepath
  (st/spec
   {:spec                (s/and string? #(re-matches #"^oss:\/\/[a-zA-Z0-9_\/]+.*" %))
    :type                :string
    :description         "File path for covertor."
    :swagger/default     nil
    :reason              "The filepath must be string."}))

(s/def ::group
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A group name which is matched with library."
    :swagger/default     []
    :reason              "The group must a string."}))

(s/def ::library
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A library name."
    :swagger/default     []
    :reason              "The library must a string."}))

(s/def ::sample
  (st/spec
   {:spec                string?
    :type                :string
    :description         "A sample name."
    :swagger/default     []
    :reason              "The sample name must a string."}))

(s/def ::metadat-item
  (s/keys :req-un [::library
                   ::group
                   ::sample]))

(s/def ::metadata
  (s/coll-of ::metadat-item))

(def quartet-rna-report-params-body
  "A spec for the body parameters."
  (s/keys :req-un [::name ::description ::filepath ::metadata]
          :opt-un [::description]))

;;; ------------------------------------------------ Event Metadata ------------------------------------------------
(def metadata
  (make-plugin-metadata
   {:name "quartet-rnaseq-report"
    :summary "Generate the QC Report for Quartet RNA-Seq data."
    :params-schema quartet-rna-report-params-body
    :handler (fn [{:keys [name description filepath metadata] :as payload}]
               (let [workdir (ff/get-workdir)
                     report-file (fs-lib/join-paths workdir "multiqc_report.html")
                     log-path (fs-lib/join-paths workdir "log")
                     response {:report report-file
                               :log log-path
                               :response-type :data2report}
                     task-id (create-task! {:name name
                                            :description (or description "Generate the QC Report for Quartet RNA-Seq data.")
                                            :payload payload
                                            :plugin-name "quartet-rnaseq-report"
                                            :plugin-type "ReportPlugin"
                                            :plugin-version "v0.1.4"
                                            :response response})]
                 (fs-lib/create-directories! workdir)
                 (spit log-path (json/write-str {:status "Running" :msg ""}))
                 (update-process! task-id 0)
                 (publish-event! "quartet_rnaseq_report"
                                 {:datadir filepath
                                  :parameters {:name name
                                               :description description
                                               :plugin-name "quartet-rnaseq-report"
                                               :plugin-version "v0.1.4"}
                                  :metadata metadata
                                  :dest-dir workdir
                                  :task-id task-id})
                 response))
    :plugin-type :ReportPlugin
    :response-type :data2report}))

;;; ------------------------------------------------ Event Processing ------------------------------------------------

(defn- decompression-tar
  [filepath]
  (shell/with-sh-env {:PATH   (add-env-to-path "quartet-rnaseq-report")
                      :LC_ALL "en_US.utf-8"
                      :LANG   "en_US.utf-8"}
    (let [command ["bash" "-c"
                   (format "tar -xvf %s -C %s" filepath (fs-lib/parent-path filepath))]
          result  (apply sh command)
          status (if (= (:exit result) 0) "Success" "Error")
          msg (str (:out result) "\n" (:err result))]
      {:status status
       :msg msg})))

(defn- filter-mkdir-copy
  [fmc-datadir fmc-patterns fmc-destdir fmc-newdir]
  (let [files-keep (fs-api/batch-filter-files fmc-datadir fmc-patterns)
        files-keep-dir (fs-lib/join-paths fmc-destdir fmc-newdir)]
    (fs-lib/create-directories! files-keep-dir)
    (fs-api/copy-files! files-keep files-keep-dir {:replace-existing true})))

(defn update-log-process!
  "Update message into log file and process into database."
  [log-path coll task-id process]
  (spit log-path (json/write-str coll))
  (update-process! task-id process))

(defn date
  []
  (.format (java.text.SimpleDateFormat. "yyyy-MM-dd")
           (new java.util.Date)))

(defn- quartet-rnaseq-report!
  "Chaining Pipeline: filter-files -> copy-files -> merge_exp_file -> exp2qcdt -> multiqc."
  [{:keys [datadir parameters metadata dest-dir task-id]}]
  (log/info "Generate quartet rnaseq report: " datadir parameters metadata dest-dir)
  (let [metadata-file (fs-lib/join-paths dest-dir
                                         "results"
                                         "metadata.csv")
        parameters-file (fs-lib/join-paths dest-dir
                                           "results"
                                           "general-info.json")
        ballgown-dir (fs-lib/join-paths dest-dir "ballgown")
        count-dir (fs-lib/join-paths dest-dir "count")
        exp-fpkm-filepath (fs-lib/join-paths dest-dir "fpkm.csv")
        exp-count-filepath (fs-lib/join-paths dest-dir "count.csv")
        result-dir (fs-lib/join-paths dest-dir "results")
        log-path (fs-lib/join-paths dest-dir "log")
        config-path (fs-lib/join-paths dest-dir "results", "quartet_rnaseq_report.yaml")]
    (try
      (fs-lib/create-directories! result-dir)
      (log/info "Merge gene experiment files from ballgown directory to a experiment table: " ballgown-dir exp-fpkm-filepath)
      (log/info "Merge gene experiment files from count directory to a experiment table: " count-dir exp-count-filepath)
      (filter-mkdir-copy datadir [".*call-ballgown/.*.txt"] dest-dir "ballgown")
      (filter-mkdir-copy datadir [".*call-count/.*gene_count_matrix.csv"] dest-dir "count")
      (filter-mkdir-copy datadir [".*call-qualimapBAMqc/.*tar.gz"] dest-dir "results/post_alignment_qc/bam_qc")
      (filter-mkdir-copy datadir [".*call-qualimapRNAseq/.*tar.gz"] dest-dir "results/post_alignment_qc/rnaseq_qc")
      (filter-mkdir-copy datadir [".*call-fastqc/.*.zip"] dest-dir "results/rawqc/fastqc")
      (filter-mkdir-copy datadir [".*call-fastqscreen/.*.txt"] dest-dir "results/rawqc/fastq_screen")
      (update-process! task-id 10)
      (me/merge-exp-files! (fs-api/list-files ballgown-dir {:mode "file"}) exp-fpkm-filepath)
      (me/merge-exp-files! (fs-api/list-files count-dir {:mode "file"}) exp-count-filepath)
      (write-csv! metadata-file metadata)
      ;;(decompression-tar files-qualimap-bam)
      ;;(decompression-tar files-qualimap-RNA)
      (doseq [files-qualimap-bam-tar (fs-api/batch-filter-files (fs-lib/join-paths dest-dir "results/post_alignment_qc/bam_qc") [".*tar.gz"])]
        (decompression-tar files-qualimap-bam-tar))
      (doseq [files-qualimap-RNA-tar (fs-api/batch-filter-files (fs-lib/join-paths dest-dir "results/post_alignment_qc/rnaseq_qc") [".*tar.gz"])]
        (decompression-tar files-qualimap-RNA-tar))
      (update-process! task-id 50)
      (gen-multiqc-config config-path)
      (let [results (ff/chain-fn-coll [(fn []
                                         (update-process! task-id 60)
                                         (exp2qcdt/call-exp2qcdt! exp-fpkm-filepath exp-count-filepath metadata-file result-dir))
                                       (fn []
                                         (update-process! task-id 70)
                                         (spit parameters-file (json/write-str {"Report Name" (:name parameters)
                                                                                "Description" (:description parameters)
                                                                                "Report Tool" (format "%s-%s"
                                                                                                      (:plugin-name parameters)
                                                                                                      (:plugin-version parameters))
                                                                                "Team" "Quartet Team"
                                                                                "Date" (date)}))
                                         {:status "Success" :msg ""})
                                       (fn []
                                         (update-process! task-id 80)
                                         (mq/multiqc result-dir dest-dir {:config config-path
                                                                          :template "quartet_rnaseq_report"
                                                                          :title "Quartet RNA report"
                                                                          :env {:PATH (add-env-to-path "quartet-rnaseq-report")}}))]
                                      (fn [result] (= (:status result) "Success")))
            status (:status (last results))
            msg (apply str (map :msg results))
            process (if (= status "Success") 100 -1)]
        (log/info (format "Running batch command: %s" (pr-str results)))
        (update-log-process! log-path {:status status
                                       :msg msg}
                             task-id process))
      (catch Exception e
        (update-process! task-id -1)
        (let [log (json/write-str {:status "Error" :msg (.toString e)})]
          (log/info "Status: " log)
          (spit log-path log))))))

;;; --------------------------------------------------- Lifecycle ----------------------------------------------------
(def events-init
  "Automatically called during startup; start event listener for quartet_rnaseq_report events."
  (make-events-init "quartet_rnaseq_report" quartet-rnaseq-report!))
